package com.uas.cloud.mall.home.service;

import com.netflix.hystrix.contrib.javanica.annotation.HystrixCommand;
import com.uas.cloud.mall.home.domain.Carousel;
import com.uas.cloud.mall.home.domain.CustomService;
import com.uas.cloud.mall.home.domain.KindInfo;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.List;

/**
 * @author yangck
 * @create 2017-02-15 17:40
 */
@Service
public class HomeService {

    private final Logger logger = LogManager.getLogger(getClass());

    @Autowired
    private RestTemplate restTemplate;

    @HystrixCommand(fallbackMethod = "getCategoryFallback")
    public List<KindInfo> getCategory(Long parentId) {
        return restTemplate.getForEntity("http://MALL-CATEGORY-SERVICE/categories?_state=withoutEmpty&parentId=" + parentId, List.class).getBody();
    }

    private List<KindInfo> getCategoryFallback(Long parentId) {
        // throw new ServiceUnavailableException("MALL-CATEGORY-SERVICE");
        logger.error("MALL-CATEGORY-SERVICE unavailable");
        return new ArrayList<>();
    }

    @HystrixCommand(fallbackMethod = "getCarouselsFallback")
    public List<Carousel> getCarousels(String usedFor) {
        return restTemplate.getForEntity("http://MALL-CAROUSEL-SERVICE/carousels?usedFor=" + usedFor, List.class).getBody();
    }

    private List<Carousel> getCarouselsFallback(String usedFor) {
        logger.error("MALL-CAROUSEL-SERVICE unavailable");
        return new ArrayList<>();
    }

    /*@HystrixCommand(fallbackMethod = "getFloors")
    public List<Floor> getFloors(String usedFor) {
        return restTemplate.getForEntity("http://MALL-FLOOR-SERVICE/floors?usedFor=" + usedFor, List.class).getBody();
    }

    public List<Floor> getFloors() {
        logger.error("MALL-FLOOR-SERVICE unavailable");
        return new ArrayList<>();
    }*/

    /**
     * 获取商城客服信息
     *
     * @param userFor  商城企业UU信息
     */
    @HystrixCommand(fallbackMethod = "getCustomService")
    public CustomService getCustomService(String userFor) {
        return restTemplate.getForEntity(String.format("http://MALL-CUSTOM-SERVICE/customService/cus/?userFor=%s", userFor), CustomService.class).getBody();
    }

    private CustomService getCustomService() {
        logger.error("MALL-CUSTOM-SERVICE unavailable");
        CustomService customService = new CustomService();
        customService.setTelephone("0755-86564892");
        customService.setAddress("深圳市南山区英唐大厦6楼");
        return customService;
    }

    /**
     * 根据用户的企业UU和用户UU获取购物车中商品的数量
     *
     * @param enuu      用户企业UU
     * @param userUU    用户UU
     * @return      购物车中商品的数量，非负
     */
    @HystrixCommand(fallbackMethod = "getCartCount")
    public Integer getCartCount(Long enuu, Long userUU) {
        Integer cartCount = restTemplate.getForEntity(String.format("http://MALL-CART-SERVICE/cart/count?enuu=%d&useruu=%d", enuu, userUU), Integer.class).getBody();
        return cartCount == null ? 0 : cartCount;
    }

    private Integer getCartCount() {
        logger.error("MALL-CART-SERVICE unavailable");
        return -1;
    }
}
